﻿Imports System.Text
Imports System.DirectoryServices

Module GetLdapAccts3_SOME

    Sub Main()

        'Dim startingPointLdapEntry As New DirectoryEntry("LDAP://ldap.ffzg.hr:636/dc=ffzg,dc=hr")
        Dim startingPointLdapEntry As New DirectoryEntry("LDAP://ldap.ffzg.hr:636/uid=pgklada,dc=ffzg,dc=hr")
        startingPointLdapEntry.AuthenticationType = AuthenticationTypes.SecureSocketsLayer  ' use ssl

        'startingPointLdapEntry.Username = Nothing                        ' anonymous binding
        'startingPointLdapEntry.Password = Nothing
        'startingPointLdapEntry.Username = "uid=pgklada,dc=ffzg,dc=hr"    ' regular user binding
        'startingPointLdapEntry.Password = "password"
        startingPointLdapEntry.Username = "cn=admin,dc=ffzg,dc=hr"        ' admin binding
        startingPointLdapEntry.Password = "password"

        ' the above just prepares a directory entry object and takes virtually no time

        Dim organizationAttributeBtw As String
        Try
            organizationAttributeBtw = startingPointLdapEntry.Properties("o")(0)  ' this actually connects to the directory and btw pulls the attribute "o" value
        Catch ex As Exception
            Console.WriteLine("Can't connect to an LDAP server. ")
            Console.WriteLine("Press any key...")
            Console.ReadKey()
            Exit Sub
        End Try

        Dim filter(0 To 12) As String
        filter(0) = Nothing
        filter(1) = "(sn=Horvat)"
        filter(2) = "(givenName=Ivana)"
        filter(3) = "(&(sn=Horvat)(givenName=Ivana))"
        filter(4) = "(sn=Horv*)"  ' Horvat, Horvatek, Horvačić, Horvatović...
        filter(5) = "(&(sn=Horv*)(!sn=Horvatek))"  ' Horvat, Horvačić, Horvatović...
        filter(6) = "(&(&(sn=Horvat)(givenName=Ivana))(!uid=ihorvat6))" ' this filter and the following get same results
        filter(7) = "(&((sn=Horvat)(givenName=Ivana)(!uid=ihorvat6)))"  ' operators work on more than two operands
        filter(8) = "(&(givenName=Ivana)(|(sn=Horvat)(sn=Krizmanić)(sn=Cencelj)))"  ' I. Horvat, I. Krizmanić, I. Cencelj
        filter(9) = "(&(givenName=Ivana)(sn=Ivan*))"  ' I. Ivančić, I. Ivanović, I. Ivandić...
        filter(10) = "(&(objectClass=*)(givenName=jasmin))"
        filter(11) = "(&(hrEduPersonRole=administrator imenika))"
        filter(12) = "(&(hrEduPersonRole=administrator imenika)(|(givenName=Luka)(givenName=Jasmin)))"

        'now we are connected and can do the rest of the job
        'walk the LDAP tree and and print the data
        Try
            ListLdapEntryAndChildren(startingPointLdapEntry, filter(0))
        Catch ex As Exception
            Console.WriteLine("Main program processing exception: " & ex.Message)
        End Try

        ' close the LDAP connection
        If startingPointLdapEntry IsNot Nothing Then
            Try
                startingPointLdapEntry.Close()
                startingPointLdapEntry.Dispose()
            Catch ex As Exception
                Console.WriteLine("Main program has failed to close the startingPointLdapEntry: " & ex.Message)
            End Try
        End If

        Console.WriteLine("")
        Console.WriteLine("Press any key...")
        Console.ReadKey()

    End Sub

    Sub ListLdapEntryAndChildren(startingPointLdapEntry As DirectoryEntry, filter As String)

        Console.WriteLine("starting from " & startingPointLdapEntry.Path)
        Console.WriteLine()

        ' declare separately, so we can close it afterwards
        Dim child As DirectoryEntry = Nothing
        Dim counter As Integer = 0

        counter += 1
        ListLdapEntry(counter, startingPointLdapEntry)

        Try
            If filter Is Nothing Then  ' no filter, just print children, if any
                For Each child In startingPointLdapEntry.Children
                    counter += 1
                    ListLdapEntry(counter, child)
                Next
            Else ' filter specified, create DicrectorySearcher, search and print found records
                Dim mySearcher As New DirectorySearcher(startingPointLdapEntry)
                mySearcher.Filter = filter
                Dim foundEntries As SearchResultCollection = mySearcher.FindAll()
                'For Each child In startingPointLdapEntry.Children
                For Each childsr As SearchResult In foundEntries
                    counter += 1
                    ListLdapEntry(counter, childsr.GetDirectoryEntry)
                Next
            End If
        Catch ex As Runtime.InteropServices.COMException
            Console.WriteLine("COM exception: " & ex.Message)
        Catch ex As Exception
            Console.WriteLine("exception: " & ex.Message)
        End Try

        ' if opened, close it
        If child IsNot Nothing Then
            child.Close()
            child.Dispose()
        End If

    End Sub

    Sub ListLdapEntry(counter As Integer, de As DirectoryEntry)

        Dim childPath As String = de.Path
        Console.WriteLine()
        Console.WriteLine()
        Console.WriteLine("{0} {1}", counter, childPath)
        Console.WriteLine()

        Static name As String = ""
        Static svalue As String = ""
        Static o As Object
        Static oib As String

        Try
            oib = Nothing
            For Each nameOrig As String In de.Properties.PropertyNames
                Try
                    o = de.InvokeGet(nameOrig)
                    name = nameOrig
                    If o Is Nothing Then
                        svalue = ""
                        name &= " (nothing)"
                    ElseIf TypeOf o Is String Then
                        Try
                            svalue = o.ToString
                            name &= " (string)"
                        Catch ex As Exception
                            svalue = ""
                            name &= " (string exception)"
                        End Try
                    ElseIf TypeOf o Is Integer Then
                        Try
                            svalue = o.ToString
                            name &= " (integer)"
                        Catch ex As Exception
                            svalue = ""
                            name &= " (integer exception)"
                        End Try
                    ElseIf TypeOf o Is System.Object() Then
                        Console.WriteLine("  > " & nameOrig & " (object array)...")
                        For n As Integer = 0 To CType(o, Object()).GetUpperBound(0)
                            Try
                                svalue = o(n).ToString
                            Catch ex As Exception
                                svalue = ""
                            End Try
                            Console.WriteLine("    >> array item " & n & ": " & svalue)
                        Next
                        Continue For ' avoid printing at the end of the loop
                    ElseIf TypeOf o Is System.Byte() Then
                        ' this code is commented out to avoid getting binary content being printed
                        'Static cvalue As Char
                        'Static sb2 As New StringBuilder
                        'sb2.Clear()
                        'For n As Integer = 0 To CType(o, Byte()).GetUpperBound(0)
                        '    Try
                        '        cvalue = Convert.ToChar(CType(o(n), Byte))
                        '    Catch ex As Exception
                        '        svalue = ""
                        '        Exit For
                        '    End Try
                        '    sb2.Append(cvalue)
                        'Next
                        'Console.WriteLine("  > " & nameOrig & " (byte array): " & sb2.ToString)
                        Continue For ' avoid printing at the end of the loop
                    Else
                        svalue = ""
                        name &= " (unknown attribute type: " & o.GetType.Name & ")"
                    End If
                Catch ex As Exception
                    If ex.Message.IndexOf("There is no such object") > -1 Then
                        Throw (New Exception("There is no such object"))
                        Exit For
                    ElseIf ex.Message.IndexOf("Logon failure") > -1 Then
                        Throw (New Exception("Logon failure"))
                        Exit For
                    ElseIf ex.Message.IndexOf("Unknown name") > -1 Then
                        svalue = ""
                    Else
                        svalue = ""
                    End If
                End Try
                Console.WriteLine("  > " & name & ": " & svalue)
            Next

        Catch ex As Runtime.InteropServices.COMException
            Console.WriteLine("COM exception: " & ex.Message)
        Catch ex As Exception
            Console.WriteLine("exception: " & ex.Message)
        End Try

        'Console.ReadKey()  ' uncomment if you want to pause after each directory entry

    End Sub

End Module
