﻿Imports System.Text
Imports System.DirectoryServices

Module GetLdapAccts4_ALL

    'Dim ldapPath As String = "LDAP://ldap.ffzg.hr:636/uid=pgklada,dc=ffzg,dc=hr"
    Dim ldapPath As String = "LDAP://ldap.ffzg.hr:636/dc=ffzg,dc=hr"
    Dim listRootEntry As Boolean = True

    'admin username is a *full ldap path* of an admin ldap entry
    Dim adminUsername = "cn=admin,dc=ffzg,dc=hr"
    'if password is not hard-coded here, it will be read from the file or from the console
    Dim adminPassword As String
    Dim adminPasswordFileName As String = My.Application.Info.DirectoryPath & "\ldap_pwd.txt"

    Dim resultsFileNameFields As String = My.Application.Info.DirectoryPath & "\ldap_accts_fields.txt"
    Dim resultsFileNameCsv As String = My.Application.Info.DirectoryPath & "\ldap_accts_csv.txt"
    Dim resultsFileNameSql As String = My.Application.Info.DirectoryPath & "\ldap_accts_sql.txt"

    'a list of fields retrieved from an LDAP server
    'if specified, than this fields are retrieved
    'if empty, all fields will be retrieved
    Dim listOfAttributesToGetFromLdapServer As String = "uid,givenName,sn,cn,displayName,hrEduPersonOIB," &
                "hrEduPersonUniqueNumber,hrEduPersonDateOfBirth,mail,hrEduPersonExpireDate," &
                "hrEduPersonPrimaryAffiliation,hrEduPersonAffiliation"
    'this collection will be filled with the list of fields asked for
    Dim soughtAttributeCollection As ICollection

    ' number of maximum number of returned LDAP entries, if 0 then all records will be listed
    Dim maxNumberOfReturnedRecords As Integer = 0

    ' csv-like output format separators
    Dim subFieldSeparator As String = "#"
    Dim subFieldSeparatorEscaped As String = "\#"
    Dim dataSeparator As String = ";"

    Dim descriptiveDataStrings As Boolean = True
    Dim writeEntriesListedToConsole As Boolean = False
    Dim writeDirectoryEntryPathsToConsole As Boolean = True

    Dim entriesListed As New StringBuilder
    'this list will receive all the data about ldap entries, each being a dictionary of pairs of strings
    Dim allLdapEntries As New List(Of Dictionary(Of String, String))

    Sub GetLdapPasswd()

        'if password is not hard-coded, read it from a file or from a console
        If adminPassword Is Nothing OrElse adminPassword.Length = 0 Then
            adminPassword = ""

            Try
                adminPassword = My.Computer.FileSystem.ReadAllText(adminPasswordFileName)
            Catch ex As Exception
            End Try

            ' if adminPasswordFile not found or is empty ask for an LDAP admin password
            If adminPassword.Length = 0 Then
                Console.WriteLine("Password for " & adminUsername)
                Console.Write("?")
                adminPassword = Console.ReadLine()
            End If
        End If

    End Sub

    Function ListAccumulatedLdapDataAsListOfFieldsAndCopyToClipboard() As String

        If entriesListed.Length = 0 Then
            My.Computer.Clipboard.SetText(" ") ' clipboard does not accept empty strings
        Else
            My.Computer.Clipboard.SetText(entriesListed.ToString)
        End If

        If writeEntriesListedToConsole Then
            Console.WriteLine(entriesListed.ToString)
        End If

        Return entriesListed.ToString

    End Function

    Function ListAccumulatedLdapDataAsLdapDataAndCopyToClipboard() As String

        Dim fieldNamesString As String =
            "uid,oib,jmbg,istek,email,rodjen,ime,prezime,imePrezime,imePrezimeBezkvako,glavniStatus,status,hrIDx"
        '            "uid,oib,jmbg,istek,email,rodjen,ime,prezime,imePrezime,imePrezimeBezkvako,glavniStatus,status,hrIDx,ldap_path"
        Dim fieldNames() As String = fieldNamesString.Split(","c)

        Dim sb As New StringBuilder
        Dim sb2 As New StringBuilder
        Dim value As String

        sb.AppendLine(fieldNamesString.Replace(","c, dataSeparator))
        For Each x In allLdapEntries
            sb2.Clear()

            For Each fieldName In fieldNames
                If sb2.Length > 0 Then sb2.Append(dataSeparator)
                If fieldName = "hrIDx" Then
                    If x.ContainsKey("hrIDx") Then
                        value = x(fieldName)
                    Else
                        value = ""
                    End If
                Else
                    value = x(fieldName)
                End If
                If value.Contains(dataSeparator) Then
                    sb2.Append("""" & value.Replace("""", """""") & """")
                Else
                    sb2.Append(value.Replace("""", """"""))
                End If
            Next
            sb.AppendLine(sb2.ToString)
        Next

        If sb.Length = 0 Then sb.Append(" ")
        My.Computer.Clipboard.SetText(sb.ToString)

        Return sb.ToString

    End Function

    Function ListAccumulatedLdapDataAsSqlInsertsAndCopyToClipboard() As String

        Dim fieldNamesString As String =
            "uid,ime,prezime,imePrezime,imePrezimeBezkvako,oib,jmbg,rodjen,email,istek,glavniStatus,status"
        Dim fieldNames() As String = fieldNamesString.Split(","c)
        Dim counter As Integer = 0
        Dim sb As New StringBuilder
        Dim sb2 As New StringBuilder

        sb.AppendLine("")
        sb.AppendLine("--DROP TABLE XLDAP")
        sb.AppendLine("")
        sb.AppendLine("CREATE TABLE [dbo].[xldap](")
        sb.AppendLine("	[uid] [nvarchar](255) NULL,")
        sb.AppendLine("	[ime] [nvarchar](255) NULL,")
        sb.AppendLine("	[prezime] [nvarchar](255) NULL,")
        sb.AppendLine("	[imePrezime] [nvarchar](255) NULL,")
        sb.AppendLine("	[imePrezimeBezkvako] [nvarchar](255) NULL,")
        sb.AppendLine("	[oib] [nvarchar](255) NULL,")
        sb.AppendLine("	[jmbg] [nvarchar](255) NULL,")
        sb.AppendLine("	[rodjen] [nvarchar](255) NULL,")
        sb.AppendLine("	[email] [nvarchar](255) NULL,")
        sb.AppendLine("	[istek] [nvarchar](255) NULL,")
        sb.AppendLine("	[glavniStatus] [nvarchar](255) NULL,")
        sb.AppendLine("	[status] [nvarchar](255) NULL,")
        sb.AppendLine("	[hrID] [nvarchar](255) NULL  -- will contain what is in hrEduPersonUniqueNumber apart from OIB and JMBG")
        sb.AppendLine(") ")
        sb.AppendLine("")
        sb.AppendLine("GO")
        sb.AppendLine("")

        ' visit all entries (saved as a list of dictionaries)
        For Each dict As Dictionary(Of String, String) In allLdapEntries
            ' chech that everyone has 14 fields and one of them is a hrID field
            If dict.Count <> 14 And Not dict.ContainsKey("hrID") Then
                Console.WriteLine("broj elemenata nije 14 nego " & dict.Count & " za korisnika " & dict.Item("uid"))
                For Each s In dict.Keys
                    Console.Write(" " & s)
                Next
                Console.WriteLine()
                Console.WriteLine("Press any key to continue...")
                Console.ReadLine()
            End If

            ' start a new INSERT statement on every 100th line
            If counter Mod 100 = 0 Then
                ' on every 100 lines, start a new INSERT statement for every hundred persons
                sb.AppendLine()
                sb.AppendLine()
                sb.AppendLine("INSERT INTO XLDAP (" & fieldNamesString & ",hrID" & ") VALUES ")
            Else
                ' otherwise just finish the current line of current INSERT statement
                sb.AppendLine(",")
            End If

            ' start a new line of data in the INSERT statement
            sb.Append("(")
            sb2.Clear()
            ' list values for all fields
            For Each fn In fieldNames
                If sb2.Length > 0 Then sb2.Append(",")
                sb2.Append("'" & dict.Item(fn) & "'")
            Next

            If dict.ContainsKey("hrID") Then
                ' add the value of hrID field
                sb2.Append(",'" & dict("hrID") & "'")
            Else
                ' or add an empty value
                sb2.Append(",''")
            End If
            ' append all collected values
            sb.Append(sb2)
            ' add the terminating parenthesis
            sb.Append(")")
            counter += 1
        Next  ' loop until all recorded values are exhausted

        sb.AppendLine("")
        sb.AppendLine("")
        sb.AppendLine("SELECT COUNT(*) FROM XLDAP")
        sb.AppendLine("")

        My.Computer.Clipboard.SetText(sb.ToString)

        Return sb.ToString

    End Function

    Function GetAttributeValuesDataString(counter As Integer, de As DirectoryEntry, childPath As String, dict As Dictionary(Of String, String)) As String

        Static sb As New StringBuilder : sb.Clear()
        Static separator As Char = "#"
        Static value As String

        sb.Append(childPath)
        For Each dicte As KeyValuePair(Of String, String) In dict
            value = dicte.Value
            value = value.Replace("#"c, "\#")
            separator = "#"c
            If dicte.Key.EndsWith(")") Then
                If dicte.Key.EndsWith("(0)") = False Then
                    separator = ","c
                End If
                value = value.Replace(","c, "\,")
            End If
            sb.Append(separator)
            sb.Append(value)
        Next

        Return sb.ToString

    End Function

    Function GetAttributeValuesString(counter As Integer, de As DirectoryEntry, childPath As String, dict As Dictionary(Of String, String)) As String

        Static sb As New StringBuilder : sb.Clear()

        sb.AppendLine("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++")
        sb.AppendLine(String.Format("  {0} {1}", counter, childPath))
        sb.AppendLine()

        For Each dicte As KeyValuePair(Of String, String) In dict
            sb.AppendLine("  " & dicte.Key & ": " & dicte.Value)
        Next

        Return sb.ToString

    End Function

    Function GetLdapEntryAttributeValuesAsDictionary(de As DirectoryEntry) As Dictionary(Of String, String)

        Static name As String = ""
        Static svalue As String = ""
        Static cvalue As Char
        Static o As Object
        Static oib As String

        Dim dict As New Dictionary(Of String, String)
        dict.Add("ldap_path", de.Path)

        Dim propertyNames As ICollection
        If listOfAttributesToGetFromLdapServer = "" Then
            propertyNames = de.Properties.PropertyNames
        Else
            propertyNames = soughtAttributeCollection
        End If

        Try
            oib = Nothing
            'For Each name As String In de.Properties.PropertyNames
            For Each nameOrig As String In propertyNames
                If nameOrig = "hrEduPersonOIB" Then
                    name = "oib"
                ElseIf nameOrig = "hrEduPersonUniqueNumber" Then
                    name = "hrID"
                ElseIf nameOrig = "hrEduPersonPrimaryAffiliation" Then
                    name = "glavniStatus"
                ElseIf nameOrig = "hrEduPersonAffiliation" Then
                    name = "status"
                ElseIf nameOrig = "hrEduPersonExpireDate" Then
                    name = "istek"
                ElseIf nameOrig = "mail" Then
                    name = "email"
                ElseIf nameOrig = "hrEduPersonDateOfBirth" Then
                    name = "rodjen"
                ElseIf nameOrig = "givenName" Then
                    name = "ime"
                ElseIf nameOrig = "sn" Then
                    name = "prezime"
                ElseIf nameOrig = "cn" Then
                    name = "imePrezime"
                ElseIf nameOrig = "displayName" Then
                    name = "imePrezimeBezkvako"
                Else
                    name = nameOrig
                End If

                Try
                    o = de.InvokeGet(nameOrig)
                    If o Is Nothing Then
                        If descriptiveDataStrings Then
                            svalue = "<no value>"
                        Else
                            svalue = ""
                        End If
                    ElseIf TypeOf o Is String Then
                        Try
                            svalue = o.ToString
                            If name = "imePrezimeBezkvako" Then
                                If svalue.IndexOfAny("čžšćđČŽŠĆĐ") >= 0 Then
                                    Console.WriteLine(name & " " & svalue & dict.Item("uid"))
                                End If
                            ElseIf name = "hrID" Then
                                AddToDictionary(dict, "hrIDx", svalue)
                            End If
                        Catch ex As Exception
                            If descriptiveDataStrings Then
                                svalue = "<not convertible>"
                            Else
                                svalue = ""
                            End If
                        End Try
                    ElseIf TypeOf o Is Integer Then
                        Try
                            svalue = o.ToString
                        Catch ex As Exception
                            If descriptiveDataStrings Then
                                svalue = "<not convertible>"
                            Else
                                svalue = ""
                            End If
                        End Try
                    ElseIf TypeOf o Is System.Object() Then
                        Static name2 As String
                        name2 = name
                        For n As Integer = 0 To CType(o, Object()).GetUpperBound(0)
                            name = name2  ' reset name at each pass of multivalued LDAP property/attribute
                            Try
                                svalue = o(n).ToString
                            Catch ex As Exception
                                If descriptiveDataStrings Then
                                    svalue = "<not convertible at element " & n & ">"
                                Else
                                    svalue = ""
                                End If
                            End Try
                            If name = "hrID" Then
                                AddToDictionary(dict, "hrIDx", svalue)
                                If svalue.StartsWith("OIB: ") Then
                                    If oib Is Nothing Then
                                        oib = svalue.Substring(5)
                                        name = "oib"
                                        svalue = oib
                                    ElseIf oib = svalue.Substring(5) Then
                                        Continue For
                                    End If
                                ElseIf svalue.StartsWith("JMBG: ") Then
                                    name = "jmbg"
                                    svalue = svalue.Substring(6)
                                End If
                                AddToDictionary(dict, name, svalue)
                                Continue For
                            Else
                                AddToDictionary(dict, name, svalue)
                            End If
                        Next
                        Continue For
                    ElseIf TypeOf o Is System.Byte() Then
                        Static sb2 As New StringBuilder
                        sb2.Clear()
                        For n As Integer = 0 To CType(o, Byte()).GetUpperBound(0)
                            Try
                                cvalue = Convert.ToChar(CType(o(n), Byte))
                            Catch ex As Exception
                                If descriptiveDataStrings Then
                                    svalue = "<byte array not convertible at element " & n & ">"
                                Else
                                    svalue = ""
                                End If
                                Exit For
                            End Try
                            sb2.Append(cvalue)
                        Next
                        AddToDictionary(dict, name & "(bytes)", sb2.ToString)
                        Continue For
                    Else
                        If descriptiveDataStrings Then
                            svalue = "unknown data type: " & o.GetType.ToString
                        Else
                            svalue = ""
                        End If
                    End If
                Catch ex As Exception
                    If ex.Message.IndexOf("There is no such object") > -1 Then
                        Throw (New Exception("There is no such object"))
                        Exit For
                    ElseIf ex.Message.IndexOf("Logon failure") > -1 Then
                        Throw (New Exception("Logon failure"))
                        Exit For
                    ElseIf ex.Message.IndexOf("Unknown name") > -1 Then
                        If descriptiveDataStrings Then
                            svalue = "Unknown field name"
                        Else
                            svalue = ""
                        End If
                    Else
                        If descriptiveDataStrings Then
                            svalue = "<not accesible: " & ex.Message & ">"
                        Else
                            svalue = ""
                        End If
                    End If
                End Try

                If name = "oib" Then
                    oib = svalue
                ElseIf name = "hrID" Then
                    If svalue.StartsWith("OIB: ") Then
                        If oib Is Nothing Then
                            oib = svalue.Substring(5)
                            name = "oib"
                            svalue = oib
                        ElseIf oib = svalue.Substring(5) Then
                            Continue For
                        End If
                    ElseIf svalue.StartsWith("JMBG: ") Then
                        name = "jmbg"
                        svalue = svalue.Substring(6)
                    End If
                End If
                AddToDictionary(dict, name, svalue)
            Next

        Catch ex As Runtime.InteropServices.COMException
            Console.WriteLine("COM exception: " & ex.Message)
        Catch ex As Exception
            Console.WriteLine("exception: " & ex.Message)
        End Try

        If dict.ContainsKey("oib") = False Then
            dict.Add("oib", "NONE")
        End If
        If dict.ContainsKey("jmbg") = False Then
            dict.Add("jmbg", "NONE")
        End If

        Return dict

    End Function

    Sub AddToDictionary(dict As Dictionary(Of String, String), name As String, value As String)

        If value = "<no value>" Then value = "NONE"

        If value.Contains(subFieldSeparator) Then
            Console.WriteLine("separator seen: " & name & " : " & value & " for uid " & dict.Item("uid"))
            value = value.Replace(subFieldSeparator, subFieldSeparator)
        End If

        If value.Contains("""") Then
            Console.WriteLine("quotes seen: " & name & " : " & value & " for uid " & dict.Item("uid"))
            value = value.Replace("""", "")
        End If

        If dict.ContainsKey(name) Then
            dict.Item(name) = dict.Item(name) & subFieldSeparator & value
        Else
            dict.Add(name, value)
        End If

    End Sub

    Delegate Function listingFunction(count As Integer, child As DirectoryEntry, childPath As String, dict As Dictionary(Of String, String)) As String

    Sub ListOneChild(counter As Integer, child As DirectoryEntry, attributeListingFunc As listingFunction)

        Dim dict As Dictionary(Of String, String)

        Dim childPath As String = child.Path
        If writeDirectoryEntryPathsToConsole Then
            Console.WriteLine("{0,5} {1}", counter, childPath)
        End If

        dict = GetLdapEntryAttributeValuesAsDictionary(child)
        allLdapEntries.Add(dict)
        entriesListed.AppendLine(attributeListingFunc(counter, child, childPath, dict))

    End Sub

    Sub ListstartingPointLdapEntryAndItsChildren(startingPointLdapEntry As DirectoryEntry, listStartingPoint As Boolean, attributeListingFunc As listingFunction)

        ' get the starting directory entry values
        entriesListed.AppendLine("")
        entriesListed.AppendLine("A list of recovered directory entries attribute values:")
        'entriesListed.AppendLine(attributeListingFunc(0, startingPointLdapEntry))

        ' declare separately, so we can close it afterwards
        Dim child As DirectoryEntry = Nothing
        Dim startTime As DateTime = DateTime.Now
        Dim counter As Integer = 0

        If listStartingPoint Then
            counter += 1
            ListOneChild(counter, startingPointLdapEntry, attributeListingFunc)
        End If

        Try
            Console.WriteLine("A list of recovered entries:")
            Console.WriteLine()
            Console.WriteLine("start " & startingPointLdapEntry.Path)
            For Each child In startingPointLdapEntry.Children
                counter += 1
                If maxNumberOfReturnedRecords <> 0 AndAlso counter > maxNumberOfReturnedRecords Then
                    Exit For ' stop if a limited number of records was needed
                End If
                ListOneChild(counter, child, attributeListingFunc)
            Next
        Catch ex As Runtime.InteropServices.COMException
            Console.WriteLine("COM exception: " & ex.Message)
        Catch ex As Exception
            Console.WriteLine("exception: " & ex.Message)
        End Try
        Console.WriteLine()
        Console.WriteLine(DateTime.Now.Subtract(startTime).TotalMilliseconds / 1000 & " seconds")
        Console.WriteLine()

        ' if opened, close it
        If child IsNot Nothing Then
            child.Close()
            child.Dispose()
        End If

    End Sub

    Function GetStartingPointLdapEntry() As DirectoryEntry

        Dim de As New DirectoryEntry(ldapPath)

        'de.Username = Nothing                          ' anonymous binding
        'de.Password = Nothing
        'de.Username = "uid=pgklada,dc=ffzg,dc=hr"      ' regular user binding
        'de.Password = "password"
        de.Username = adminUsername
        de.Password = adminPassword

        de.AuthenticationType = AuthenticationTypes.SecureSocketsLayer  ' use ssl

        Return de

    End Function

    Sub Main()

        GetLdapPasswd()  ' get LDAP admin password , either from a file or from console
        Dim startingPointLdapEntry As DirectoryEntry = GetStartingPointLdapEntry()  ' this just prepares a directory entry object and takes virtually no time

        Dim startTime As DateTime = DateTime.Now

        Dim rootPath As String
        Try
            rootPath = startingPointLdapEntry.Properties("o")(0)  ' this actually connects to the directory and btw pulls the attribute "o" value
        Catch ex As Exception
            Console.WriteLine("Can't connect to an LDAP server. ")
            Console.WriteLine("Press any key...")
            Console.ReadKey()
            Exit Sub
        End Try

        Console.WriteLine("Connecting time: " & DateTime.Now.Subtract(startTime).ToString & " seconds")
        Console.WriteLine()

        'now we are connected and can do the rest of the job

        'list all attributes or just enlisted ones?
        If listOfAttributesToGetFromLdapServer <> "" Then
            soughtAttributeCollection = listOfAttributesToGetFromLdapServer.Split(","c)
        End If

        'walk the same LDAP tree and use on or the other function to prepare the data
        Try
            If descriptiveDataStrings Then
                ListstartingPointLdapEntryAndItsChildren(startingPointLdapEntry, listRootEntry, AddressOf GetAttributeValuesString)
            Else
                ListstartingPointLdapEntryAndItsChildren(startingPointLdapEntry, listRootEntry, AddressOf GetAttributeValuesDataString)
            End If
        Catch ex As Exception
            Console.WriteLine("Main program processing exception: " & ex.Message)
        End Try

        ' close the LDAP connection
        If startingPointLdapEntry IsNot Nothing Then
            Try
                startingPointLdapEntry.Close()
                startingPointLdapEntry.Dispose()
            Catch ex As Exception
                Console.WriteLine("Main program has failed to close the startingPointLdapEntry: " & ex.Message)
            End Try
        End If

        'turn collected LDAP data into output
        Dim resultCsv As String = ListAccumulatedLdapDataAsLdapDataAndCopyToClipboard()
        Dim resultFields As String = ListAccumulatedLdapDataAsListOfFieldsAndCopyToClipboard()
        Dim resultSql As String = ListAccumulatedLdapDataAsSqlInsertsAndCopyToClipboard()

        'write generated output to file
        My.Computer.FileSystem.WriteAllText(resultsFileNameCsv, resultCsv, False)
        My.Computer.FileSystem.WriteAllText(resultsFileNameFields, resultFields, False)
        My.Computer.FileSystem.WriteAllText(resultsFileNameSql, resultSql, False)

        Console.WriteLine("")
        Console.WriteLine("")
        Console.WriteLine("Press any key...")

        Console.ReadKey()

    End Sub

End Module
